/*******************************************************************************
* File Name: CNTR_IR_Capture_PM.c  
* Version 3.0
*
*  Description:
*    This file provides the power management source code to API for the
*    Counter.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "CNTR_IR_Capture.h"

static CNTR_IR_Capture_backupStruct CNTR_IR_Capture_backup;


/*******************************************************************************
* Function Name: CNTR_IR_Capture_SaveConfig
********************************************************************************
* Summary:
*     Save the current user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  CNTR_IR_Capture_backup:  Variables of this global structure are modified to 
*  store the values of non retention configuration registers when Sleep() API is 
*  called.
*
*******************************************************************************/
void CNTR_IR_Capture_SaveConfig(void) 
{
    #if (!CNTR_IR_Capture_UsingFixedFunction)

        CNTR_IR_Capture_backup.CounterUdb = CNTR_IR_Capture_ReadCounter();

        #if(!CNTR_IR_Capture_ControlRegRemoved)
            CNTR_IR_Capture_backup.CounterControlRegister = CNTR_IR_Capture_ReadControlRegister();
        #endif /* (!CNTR_IR_Capture_ControlRegRemoved) */

    #endif /* (!CNTR_IR_Capture_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CNTR_IR_Capture_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  CNTR_IR_Capture_backup:  Variables of this global structure are used to 
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void CNTR_IR_Capture_RestoreConfig(void) 
{      
    #if (!CNTR_IR_Capture_UsingFixedFunction)

       CNTR_IR_Capture_WriteCounter(CNTR_IR_Capture_backup.CounterUdb);

        #if(!CNTR_IR_Capture_ControlRegRemoved)
            CNTR_IR_Capture_WriteControlRegister(CNTR_IR_Capture_backup.CounterControlRegister);
        #endif /* (!CNTR_IR_Capture_ControlRegRemoved) */

    #endif /* (!CNTR_IR_Capture_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: CNTR_IR_Capture_Sleep
********************************************************************************
* Summary:
*     Stop and Save the user configuration
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  CNTR_IR_Capture_backup.enableState:  Is modified depending on the enable 
*  state of the block before entering sleep mode.
*
*******************************************************************************/
void CNTR_IR_Capture_Sleep(void) 
{
    #if(!CNTR_IR_Capture_ControlRegRemoved)
        /* Save Counter's enable state */
        if(CNTR_IR_Capture_CTRL_ENABLE == (CNTR_IR_Capture_CONTROL & CNTR_IR_Capture_CTRL_ENABLE))
        {
            /* Counter is enabled */
            CNTR_IR_Capture_backup.CounterEnableState = 1u;
        }
        else
        {
            /* Counter is disabled */
            CNTR_IR_Capture_backup.CounterEnableState = 0u;
        }
    #else
        CNTR_IR_Capture_backup.CounterEnableState = 1u;
        if(CNTR_IR_Capture_backup.CounterEnableState != 0u)
        {
            CNTR_IR_Capture_backup.CounterEnableState = 0u;
        }
    #endif /* (!CNTR_IR_Capture_ControlRegRemoved) */
    
    CNTR_IR_Capture_Stop();
    CNTR_IR_Capture_SaveConfig();
}


/*******************************************************************************
* Function Name: CNTR_IR_Capture_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  CNTR_IR_Capture_backup.enableState:  Is used to restore the enable state of 
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void CNTR_IR_Capture_Wakeup(void) 
{
    CNTR_IR_Capture_RestoreConfig();
    #if(!CNTR_IR_Capture_ControlRegRemoved)
        if(CNTR_IR_Capture_backup.CounterEnableState == 1u)
        {
            /* Enable Counter's operation */
            CNTR_IR_Capture_Enable();
        } /* Do nothing if Counter was disabled before */    
    #endif /* (!CNTR_IR_Capture_ControlRegRemoved) */
    
}


/* [] END OF FILE */
