/* ========================================
 *
 * Copyright Wade Maxfield, 2020
 * All Rights Reserved
 * LICENSED SOFTWARE.
 * WHICH IS THE PROPERTY OF Wade Maxfield.
 * Released under the MIT license
 * ========================================
*/

/* [] END OF FILE */

#include <ds18b20.h>
#include <stdio.h>

//--------------------------------------------
// These defines are used to make it easier
// to change pin names.
//--------------------------------------------
#define OneWire_Read    P3_5_OneWire_Read
#define OneWire_Write   P3_5_OneWire_Write

uint8   OneWireResetResults;
char BUFFER[16] = " " ;  // buffer for 6 bytes SERIAL NUMBER
uint16  OneWireError;


//--------------------------------------------
//--------------------------------------------
void initialize_1_wire_device()
{
    uint16 ReadHEX ;                // Read byte from 1-WIRE
     
    OneWireResetResults = Reset1WireBus();
    Write1WireByte(0x33);             
    ReadHEX = Read1WireByte();
    // flag an error condition if it occured
    if (ReadHEX != 0x28 && ReadHEX != 0x10)
        OneWireError=1; 
        
}
//--------------------------------------------
//--------------------------------------------
uint8 Read1WireBit() //***  Read a single bit off the 1-WIRE bus. ***
{
    CyDelayUs(20);  
    OneWire_Write(0);    // signal want to read
    CyDelayUs(TIMEOUT_READ_WRITE_1_BIT); 
    OneWire_Write(1);    // send pin high so can read device response on line 
    CyDelayUs(5);       
 	return(OneWire_Read());    

}
//--------------------------------------------
//--------------------------------------------
void Write1WireBit(uint8 payload) // Write a single bit to the 1-WIRE bus. 
{
    CyDelayUs(20);  //Inactive period
    OneWire_Write(0);       
    
    if(payload==0)
        CyDelayUs(TIMEOUT_READ_WRITE_0_BIT); //0 write
    else     		    
        CyDelayUs(TIMEOUT_READ_WRITE_1_BIT); //1 write
        
    OneWire_Write(1);       //Send High
}
//--------------------------------------------
//--------------------------------------------
uint8 Reset1WireBus() 
{
    uint8 BusPin=0 ;      
    
    OneWire_Write(0);  
    
    CyDelayUs(WAIT_TIME_US); 
    
    OneWire_Write(1);     
    CyDelayUs(5);      

    if (OneWire_Read()==0)  	
        return(0xFF);   //the line is shorted, exit with error indication.
        
    CyDelayUs(55);      
    
    // read the value on the pin
    BusPin = OneWire_Read();   
    CyDelayUs(200);     // wait for temperature sensor to be done

    return(BusPin);       
} 
//-----------------------------------------------------------------------------
// Write a byte to the serial bus
//-----------------------------------------------------------------------------
void Write1WireByte(uint8 byteToWrite) 
{
    uint8  shiftcount;
    
    for (shiftcount=0; shiftcount<=7;shiftcount++) {
        Write1WireBit((byteToWrite >> shiftcount) & 0x01);
    }
}      
//-----------------------------------------------------------------------------
// Return a byte read from the serial bus
//-----------------------------------------------------------------------------
uint8 Read1WireByte() 
{
	uint8 IncomingByte=0, shiftcount=0 ;
	for (shiftcount=0; shiftcount<=7; shiftcount++) 
    {
       IncomingByte |= (Read1WireBit())<<shiftcount;//let's start with LSB
    CyDelayUs(6);
    }
    return(IncomingByte);
}
//-----------------------------------------------------------------------------
// Temperature reading.
//-----------------------------------------------------------------------------
float celcius=25.0;

//-----------------------------------------------------------------------------
// we use a state machine to direct the process of reading the temperature.
//-----------------------------------------------------------------------------
enum TemperatureReadState {
    nothing=0
    ,start
    ,waitForRBit
    ,readTemperature
    ,readStaticRamFromTempSensor
    ,tempAvailable
};
enum TemperatureReadState TemperatureState=nothing;

//-----------------------------------------------------------------------------
// Return the temperature in celcius
//-----------------------------------------------------------------------------
float getTemperature()
{
    static int16 i;
    static uint8 staticRamBytes[8];
    int16_t a2dReading;
    
    switch (TemperatureState){
        case nothing:
            TemperatureState = start;
        
        case start:         
            Reset1WireBus();
            Write1WireByte(Skip_ROM);		
        	Write1WireByte(BeginTempConvertion);
            TemperatureState = waitForRBit;
        break;

            // NOTE: we don't have to wait for any time 
            // to read this, since we are always powering the device
        case waitForRBit:
            //CyDelayUs(820);
	        if (!Read1WireBit())
                break; 
            // else
            TemperatureState = readTemperature;
            
        break;
            
        case readTemperature:
        	Reset1WireBus();
        	Write1WireByte(Skip_ROM);		
        	Write1WireByte(Read_scratchpad);	
	
            CyDelayUs(1);
            TemperatureState = readStaticRamFromTempSensor;
            i=0;
        break;
            
        case readStaticRamFromTempSensor:
            staticRamBytes[i]=Read1WireByte();
            i++;
            if (i<8)
                break;
				
           
            a2dReading = (staticRamBytes[1] << 8) | staticRamBytes[0];
 
            a2dReading  <<= 3; // 9 bit resolution default
            
            //if sensor is ds1820 or ds18s20 only 9 bit 
            // calculations. See datasheet!
            if (staticRamBytes[7] == 0x10){ 
              // remaining count gives full 12 bit resolution
              a2dReading = (a2dReading & 0xFFF0) + 12 - staticRamBytes[6];
            } else { 
                //if sensor is Ds18B20 9 through 12 bits are available
                int8 configuration = (staticRamBytes[4] & 0x60);
                // if lower res, the low bits are undefined
                switch (configuration) {
                    case 0:
                        a2dReading &= ~7;  // 9 bits,takes 93.75 ms
                    break;
                    
                    case 0x20:
                       a2dReading &= ~3; // 10 bits, takes 187.5 ms
                    break;
                    
                    case 0x40:
                        a2dReading &= ~1;  // 11 bits, takes  375 ms
                    break;
                    
                    default: 
                        // default is 12 bit resolution, 
                        // takes a full 750 ms conversion time
                        // so we need to time-slice readings so as 
                        // to not freeze the rest of the system
                    break;
                }// switch
            }//else

        
            celcius=a2dReading/160.0f;
            TemperatureState = tempAvailable;
        break;
            
        case tempAvailable:
            // hang in this state until we are restarted
        break;
    }
    
    // return old values until new value exists
    return celcius;  
         
}
float TempF=70.0;

//-----------------------------------------------------------------------------
// return the temperature in degrees F
//-----------------------------------------------------------------------------
float getTemperatureF(int16 *readingAvailable){
    
    // if a non-zero passed in, assume we can write to it
    if (readingAvailable)
        *readingAvailable=0;// indicate no reading is available
    
    getTemperature();//run the state machine, ignore return
    
    if (TemperatureState == tempAvailable){
        TemperatureState = nothing; // restart reading next time    
        TempF = celcius *1.8f+32.0f;// we are using the global, could use return
        // indicate we have a reading
        if (readingAvailable)
           *readingAvailable = 1;
    }
    return TempF;
    
}

